using System;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using Eco.Handles;
using Eco.Windows.Forms;
using Eco.WinForm;
using Eco.ObjectRepresentation;
using Eco.Services;

namespace ECO_ORMBattle_SimpleExample
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public partial class Form1 : System.Windows.Forms.Form
	{
        private bool FThreadExit = false;
        private bool FThreadException = false;
        private Thread FEcoSync = null;

        public void DoEcoSpaceSync()
        {
            try
            {
                if (FThreadException)
                    return;

                while (!FThreadExit)
                {
                    if (EcoSpace != null)
                    {
                        if (EcoSpace.Persistence != null)
                        {
                            EcoSpace.Persistence.RetrieveChanges();

                            IChange[] RecentChanges = EcoSpace.Persistence.GetChanges();
                            if (RecentChanges.Length > 0)
                            {
                                try
                                {
                                    foreach (IChange change in RecentChanges)
                                    {
                                        if (change.IsDirectConflict)
                                        {
                                            // Conflict resolution??
                                            change.Action = ChangeActionKind.Keep;
                                        }
                                        else
                                        {
                                            change.Action = ChangeActionKind.Discard;
                                        }
                                    }
                                }
                                finally
                                {
                                    EcoSpace.Persistence.ApplyAllChanges();
                                }
                            }
                            else
                            {
                                // No recent change to concern ourselves with
                            }
                        }
                        else
                        {
                            // Error!
                            // We have no access to the Persistence Service!  We may have a big issue here!
                        }
                    }
                    else
                    {
                        // Error!
                        // We have no EcoSpace to work with
                    }

                    Thread.Sleep(2000);
                }
            }
            catch (Exception)
            {
                // do nothing, just exit
                FThreadException = true;
            }
        }


		public Form1(ECO_ORMBattle_SimpleExample.ECO_ORMBattle_SimpleExampleEcoSpace ecoSpace)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Set EcoSpace (actually stores it in rhRoot.EcoSpace)
			// Note that you need to set the EcoSpace property for 
			// each ReferenceHandle and VariableHandle you add to 
			// the form.
			rhRoot.EcoSpace = ecoSpace;


            FEcoSync = new Thread((ThreadStart)this.DoEcoSpaceSync);
            FEcoSync.IsBackground = true;
            FEcoSync.Priority = ThreadPriority.Normal;
            // Start after it's first shown
            FEcoSync.Start();
		}

		public ECO_ORMBattle_SimpleExample.ECO_ORMBattle_SimpleExampleEcoSpace EcoSpace
		{
			get { return (ECO_ORMBattle_SimpleExample.ECO_ORMBattle_SimpleExampleEcoSpace)rhRoot.EcoSpace; }
		}

        private void button1_Click(object sender, EventArgs e)
        {
            ManageVatRates mvr = new ManageVatRates(EcoSpace);
            mvr.ShowDialog();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Update the database before we do anything else.
            if (this.EcoSpace.DirtyList.HasDirtyObjects())
            {
                this.EcoSpace.UpdateDatabase();
            }

            if (FEcoSync != null)
            {
                if (FEcoSync.ThreadState != ThreadState.Stopped)
                {
                    try
                    {
                        FThreadExit = true;
                        //FEcoSync.Abort();
                    }
                    catch (Exception)
                    {
                        //throw;
                    }
                }
            }
        }

        private void btnManageInvoices_Click(object sender, EventArgs e)
        {
            ManageInvoices mi = new ManageInvoices(EcoSpace);
            mi.ShowDialog();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            SimpleMasterDetailExample simple = new SimpleMasterDetailExample(EcoSpace);
            simple.ShowDialog();
        }
	}
}
